import os
import urllib.request
import threading
import hvac

default_vault_address = os.getenv('VAULT_ADDR', 'http://127.0.0.1:8200')


class Vault:
    vault_server = default_vault_address
    connection_mode = 'token'

    _client = None
    _instance = None
    _lock = threading.Lock()

    parameters = {}

    @classmethod
    def instance(cls, **kwargs):
        if Vault._instance is None:
            with Vault._lock:
                if Vault._instance is None:
                    Vault._instance = Vault(**kwargs)
        return Vault._instance

    def __init__(self, server=default_vault_address, connection_mode='token', client=None, **kwargs):
        self.connection_mode = connection_mode
        self.vault_server = server
        self.parameters = kwargs
        self._client = client

    @property
    def client(self):
        if self._client is not None:
            if self._client.is_authenticated():
                return self._client
            else:
                # noinspection PyBroadException
                try:
                    self._client.close()
                except:
                    pass

        client = hvac.Client(url=self.vault_server)
        if self.connection_mode == 'token':
            client.token = self.parameters['token']
        elif self.connection_mode == 'ec2':
            contents = urllib.request.urlopen('http://169.254.169.254/latest/dynamic/instance-identity/pkcs7')\
                .read() \
                .decode('utf-8') \
                .replace('\n', '')
            client.auth_ec2(contents, nonce=self.parameters['nonce'], role=self.parameters['role'], mount_point='aws')
        else:
            getattr(client, 'auth_' + self.connection_mode)(**self.parameters)

        self._client = client
        return self._client



import psycopg2


class VaultPsycopg:
    _vault_wrapper = None
    _database_connection = None

    __username = None
    __password = None

    database_config = {}
    vault_config = {}

    def __init__(self, database_config=None, vault_config=None):
        self.vault_config = vault_config
        self.database_config = database_config
        if vault_config:
            from vault_psycopg2.vault import Vault
            self._vault_wrapper = Vault.instance(**vault_config)

    @property
    def credentials(self):
        _ = self.connection
        return {
            "username": self.__username,
            "password": self.__password
        }

    @property
    def connection(self):
        if self._database_connection:
            try:
                cursor = self._database_connection.cursor()
                cursor.execute('SELECT 1')
                return self._database_connection
            except psycopg2.Error:
                pass

        if self._vault_wrapper and self.database_config.get('role') is not None:
            credentials = self._vault_wrapper.client.read('database/creds/' + self.database_config.get('role'))
            self.__username = credentials['data']['username']
            self.__password = credentials['data']['password']
        else:
            self.__username = self.database_config.get('user')
            self.__password = self.database_config.get('password')

        self._database_connection = psycopg2.connect(
            host=self.database_config.get('host'),
            port=self.database_config.get('port'),
            dbname=self.database_config.get('dbname'),
            user=self.__username,
            password=self.__password,
        )

        return self._database_connection



#!/usr/bin/env python
# -* encoding: utf-8 *-
import os
from setuptools import setup

HERE = os.path.dirname(__file__)

try:
    long_description = open(os.path.join(HERE, 'README.md')).read()
except IOError:
    long_description = None


setup(
    name="vault-psycopg2",
    version="0.1.11",
    packages=["vault_psycopg2"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    url="https://github.com/niranjan94/vault-psycopg2",
    author="Niranjan Rajendran (@niranjan94)",
    author_email="me@niranjan.io",
    maintainer="Niranjan Rajendran (@niranjan94)",
    maintainer_email="me@niranjan.io",
    description="Helper classes to integrate psycopg2 with Vault",
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=[
        "hvac>=0.3.0",
        "psycopg2-binary>=2.0.0",
    ],
)
